package sun.management.snmp.jvminstr;

import java.lang.management.ManagementFactory;
import java.util.List;
import sun.management.snmp.util.SnmpNamedListTableCache;
import com.sun.jmx.mbeanserver.Util;
import java.util.Map;
import java.lang.management.MemoryManagerMXBean;
import sun.management.snmp.util.JvmContextFactory;
import sun.management.snmp.util.SnmpTableHandler;
import com.sun.jmx.snmp.SnmpStatusException;
import com.sun.jmx.snmp.SnmpOid;
import com.sun.jmx.snmp.agent.SnmpStandardObjectServer;
import com.sun.jmx.snmp.agent.SnmpMib;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.util.SnmpTableCache;
import sun.management.snmp.jvmmib.JvmMemManagerTableMeta;

public class JvmMemManagerTableMetaImpl extends JvmMemManagerTableMeta
{
    static final long serialVersionUID = 36176771566817592L;
    protected SnmpTableCache cache;
    static final MibLogger log;
    
    public JvmMemManagerTableMetaImpl(SnmpMib snmpMib, SnmpStandardObjectServer snmpStandardObjectServer) {
        super(snmpMib, snmpStandardObjectServer);
        this.cache = new JvmMemManagerTableMetaImpl.JvmMemManagerTableCache(((JVM_MANAGEMENT_MIB_IMPL)snmpMib).validity());
    }
    
    protected SnmpOid getNextOid(Object o) throws SnmpStatusException {
        return this.getNextOid(null, o);
    }
    
    protected SnmpOid getNextOid(SnmpOid obj, Object o) throws SnmpStatusException {
        int debugOn = JvmMemManagerTableMetaImpl.log.isDebugOn();
        if (!(debugOn == 0)) {
            JvmMemManagerTableMetaImpl.log.debug("getNextOid", "previous=" + obj);
        }
        SnmpTableHandler handler = this.getHandler(o);
        if (handler == null) {
            if (!(debugOn == 0)) {
                JvmMemManagerTableMetaImpl.log.debug("getNextOid", "handler is null!");
            }
            throw new SnmpStatusException(224);
        }
        else {
            SnmpOid next = handler.getNext(obj);
            if (!(debugOn == 0)) {
                JvmMemManagerTableMetaImpl.log.debug("getNextOid", "next=" + next);
            }
            if (next == null) {
                throw new SnmpStatusException(224);
            }
            return next;
        }
    }
    
    protected boolean contains(SnmpOid snmpOid, Object o) {
        SnmpTableHandler handler = this.getHandler(o);
        return handler != null && handler.contains(snmpOid);
    }
    
    public Object getEntry(SnmpOid snmpOid) throws SnmpStatusException {
        if (snmpOid == null) {
            throw new SnmpStatusException(224);
        }
        Map<String, JvmMemManagerEntryImpl> userData = JvmContextFactory.getUserData();
        long oidArc = snmpOid.getOidArc(0);
        String s = (userData == null) ? null : ("JvmMemManagerTable.entry." + oidArc);
        if (userData != null) {
            Object value = userData.get(s);
            if (value != null) {
                return value;
            }
        }
        SnmpTableHandler handler = this.getHandler(userData);
        if (handler == null) {
            throw new SnmpStatusException(224);
        }
        Object data = handler.getData(snmpOid);
        if (data == null) {
            throw new SnmpStatusException(224);
        }
        JvmMemManagerEntryImpl jvmMemManagerEntryImpl = new JvmMemManagerEntryImpl((MemoryManagerMXBean)data, (int)oidArc);
        if (userData != null && jvmMemManagerEntryImpl != null) {
            userData.put(s, jvmMemManagerEntryImpl);
            return jvmMemManagerEntryImpl;
        }
        return jvmMemManagerEntryImpl;
    }
    
    protected SnmpTableHandler getHandler(Object x) {
        Map<String, SnmpTableHandler> map;
        if ((x instanceof Map)) {
            map = (Map)Util.cast(x);
        }
        else {
            map = null;
        }
        if (map != null) {
            SnmpTableHandler snmpTableHandler = (SnmpTableHandler)map.get("JvmMemManagerTable.handler");
            if (snmpTableHandler != null) {
                return snmpTableHandler;
            }
        }
        SnmpTableHandler tableHandler = this.cache.getTableHandler();
        if (map != null && tableHandler != null) {
            map.put("JvmMemManagerTable.handler", tableHandler);
            return tableHandler;
        }
        return tableHandler;
    }
    
    static {
        log = new MibLogger(JvmMemManagerTableMetaImpl.class);
    }
    
    private static class JvmMemManagerTableCache extends SnmpNamedListTableCache
    {
        static final long serialVersionUID = 6564294074653009240L;
        
        JvmMemManagerTableCache(long validity) {
            this.validity = validity;
        }
        
        protected String getKey(Object o, List<?> list, int n, Object o2) {
            if (o2 == null) {
                return null;
            }
            String name = ((MemoryManagerMXBean)o2).getName();
            JvmMemManagerTableMetaImpl.log.debug("getKey", "key=" + name);
            return name;
        }
        
        public SnmpTableHandler getTableHandler() {
            return this.getTableDatas(JvmContextFactory.getUserData());
        }
        
        protected String getRawDatasKey() {
            return "JvmMemManagerTable.getMemoryManagers";
        }
        
        protected List<MemoryManagerMXBean> loadRawDatas(Map<Object, Object> map) {
            return ManagementFactory.getMemoryManagerMXBeans();
        }
    }
}
